import os
from gettext import *

from pkg_resources import resource_stream

def egg_translation(domain, lang):
    """
    This method doesn't do all the checking etc of the gettext.translation method
    but it seems to work.
    
    We can't just use gettext.translation because the .mo files might be in eggs
    """
    class_ = GNUTranslations
    return class_(resource_stream(domain, os.path.join(lang, '%s.mo' % domain)))



"""Legacy objects for Pylons 0.8 projects

Myghty compatibility object, old-style ``params`` and ``m`` globals. The
``response`` object is used to buffer the output.
"""
import types
import sys
import warnings

from paste.registry import StackedObjectProxy
import paste.httpexceptions as httpexceptions

import pylons
import pylons.templating as tmpl

# Legacy objects
m = StackedObjectProxy(name="m legacy object")
response = StackedObjectProxy(name="response")
params = StackedObjectProxy(name="params")

def load_h(package_name):
    """
    This is a legacy test for pre-0.9.3 projects to continue using the old
    style Helper imports. The proper style is to pass the helpers module ref
    to the PylonsApp during initialization.
    """
    warnings.warn(
        "Pylons 0.9.3 and above now passes the helpers module reference in "
        "directly. It's highly recommended that you update your middleware.py "
        "module so that the Pylons app load looks similar to:\n"
        "app = pylons.wsgiapp.PylonsApp(config, helpers=MYPROJ.lib.helpers, "
        "g=app_globals.Globals)\n"
        "Where MYPROJ is the name of your project. Also make sure to add an "
        "import line to the middleware.py to import the helpers modules:\n"
        "import MYPROJ.lib.helpers\n"
        "This will be required on all projects 1.0 and beyond.",
        DeprecationWarning, 2)
    __import__(package_name + '.lib.base')
    their_h = getattr(sys.modules[package_name + '.lib.base'], 'h', None)
    if isinstance(their_h, types.ModuleType):
        # lib.base.h is a module (and thus not pylons.h) -- assume lib.base uses
        # new style (self contained) helpers via:
        # import ${package}.lib.helpers as h
        return their_h

    # Assume lib.base.h is a StackedObjectProxy -- lib.base is using pre 0.9.2
    # style helpers via:
    # from pylons import h
    try:
        helpers_name = package_name + '.lib.helpers'
        __import__(helpers_name) 
    except ImportError:
        # pylons 0.8.x support
        helpers_name = package_name + '.config.helpers'
        __import__(helpers_name)
    helpers_module = sys.modules[helpers_name]

    # Pre 0.9.2 lib.helpers did not import the pylons helper functions, manually
    # add them. Don't overwrite user functions (allowing pylons helpers to be
    # overridden)
    for func_name in ('_', 'log', 'set_lang', 'get_lang'):
        if not hasattr(helpers_module, func_name):
            setattr(helpers_module, func_name, getattr(pylons.util, func_name))

    return sys.modules[helpers_name]
    
class MyghtyCompat(object):
    """Myghty Compatibility Object for Pylons 0.8 Projects"""
    def __init__(self, environ, start_response):
        """Create Myghty compatibility object given WSGI interface"""
        self.environ = environ
        self.start_response = start_response
        self.headers_out = response.headers
        self.headers_in = pylons.request.headers
        self.request_args = pylons.request._legacy_params
    
    def write(self, content):
        """Write content to the response buffer"""
        warnings.warn(
            "The m.write call has been deprecated. Replace with "
            "response.write() and instantiate a Response object.",
            DeprecationWarning, 2)
        response.write(content)
    out = write
    
    def cache_self(self, *args, **kargs):
        """Pretends to cache itself, actually disables caching"""
        warnings.warn(
            "The m.cache_self call has been deprecated. Replace usage of "
            "m.cache_self with return render_response() and add cache "
            "keywords.",
            DeprecationWarning, 2)
        return False
    
    def subexec(self, *args, **kargs):
        """Renders a template to the response buffer"""
        warnings.warn(
            "The m.subexec call has been deprecated. Replace with "
            "return render_response().",
            DeprecationWarning, 2)
        response.write(tmpl.render(*args, **kargs))
    
    def comp(self, *args, **kargs):
        """Renders a template fragment to the response buffer"""
        warnings.warn(
            "The m.comp call has been deprecated. Replace with return "
            "render_response('sometemplate.myt', fragment=True) or "
            "response.write(render('sometemplate.myt', fragment=True)) "
            "as necessary.",
            DeprecationWarning, 2)
        response.write(tmpl.render(fragment=True, *args, **kargs))
    
    def scomp(self, *args, **kargs):
        """Returns a rendered template fragment as a string"""
        warnings.warn(
            "The m.scomp call has been deprecated. Replace with result = "
            "render('sometemplate.myt', fragment=True).",
            DeprecationWarning, 2)
        return tmpl.render(fragment=True, *args, **kargs)
    
    def fetch_component(self, name):
        """Returns the name of the component"""
        warnings.warn(
            "The m.fetch_component call has been deprecated. See cache docs.",
            DeprecationWarning, 2)
        return name
    
    def get_cache(self, component):
        """Returns a cache namespace"""
        warnings.warn(
            "The m.get_cache call has been deprecated. See cache docs.",
            DeprecationWarning, 2)
        return pylons.cache.get_cache(component)
    
    def send_redirect(self, path, hard=True):
        """Raises a redirect exception for Paste HTTPExceptions"""
        warnings.warn(
            "The m.send_redirect call has been deprecated. Replace with "
            "redirect_to()",
            DeprecationWarning, 2)
        raise httpexceptions.HTTPFound(path)
    
    def abort(self, status_code=None, reason=""):
        """Raises a 404 or the given exception for Paste HTTPExceptions"""
        warnings.warn(
            "The m.abort call has been deprecated. Replace with "
            "abort(200, detail='some message').",
            DeprecationWarning, 2)
        if status_code == 404:
            raise httpexceptions.HTTPNotFound()
        else:
            exc = httpexceptions.get_exception(status_code)(detail=reason)
            raise exc

__all__ = ['m', 'params']



"""
Setuptools extensions and tools to add multiple language support
"""
import setuptools, sys, os, re

# Ignore these directories when iterating through paths
exclude_dirs = ('.AppleDouble', '.svn', 'CVS', '_darcs')

class LangExtract(setuptools.Command):

    description = "extract strings for internationalisation"
    user_options = []
    boolean_options = []
    def initialize_options(self): pass
    def finalize_options(self): pass
        
    def run(self):
        project = re.compile('[^a-zA-Z0-9_]').sub('', self.distribution.get_name().lower())
        i18n_path = os.path.join(project, 'i18n%s' % os.sep)
        pot_filename = '%s.pot' % project
        pot_path = os.path.join(i18n_path, pot_filename)
        
        if os.path.exists(pot_path):
            print "Error: File %s already exists" % pot_path
            sys.exit(1)

        oldsys = sys.argv
        sys.argv.pop(1)
        sys.argv.extend(['-K', '-k', '_', '-o', pot_filename, '-p', i18n_path, '*'])
        
        import pylons.i18n.pygettext
        pylons.i18n.pygettext.pot_header = '''\
# Pylons Project %s Translation File
# Copyright (C) YEAR ORGANIZATION
# FIRST AUTHOR <EMAIL@ADDRESS>, YEAR.
#
msgid ""
msgstr ""
"Project-Id-Version: PACKAGE VERSION\\n"
"POT-Creation-Date: %%(time)s\\n"
"PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\\n"
"Last-Translator: %s <%s>\\n"
"Language-Team: LANGUAGE <LL@li.org>\\n"
"MIME-Version: 1.0\\n"
"Content-Type: text/plain; charset=UTF-8\\n"
"Content-Transfer-Encoding: ENCODING\\n"
"Generated-By: Pylons %%(version)s\\n"

''' % (
        project,
        self.distribution.get_author().lower(),
        self.distribution.get_author_email().lower(), 
        )
        # These could be filled in from setup.py
        pylons.i18n.pygettext.main()
        print "\nSucessfully created langauage template in %s\n" % pot_path
        print """Now create your langauge files, save them in i18n/lang replacing 
lang with the language code and changing the file extension to .po then run 
setup.py lang_compile mode to produce your .mo files"""

        sys.argv = oldsys

class LangCompile(setuptools.Command):
    description = "compile strings for internationalisation"
    user_options = [('lang=', 'l', "language to compile"),]
    boolean_options = []
    def finalize_options(self): pass

    def initialize_options(self): 
        self.lang = None
        
    def compile_lang(self, lang):
        project = re.compile('[^a-zA-Z0-9_]').sub('', self.distribution.get_name().lower())
        lang_path = os.path.join(project, 'i18n', lang)
        po_path = os.path.join(lang_path, '%s.po' % project)
        lc_path = os.path.join(lang_path, 'LC_MESSAGES')
        mo_path = os.path.join(lc_path, '%s.mo' % project)

        if not os.path.exists(lang_path) or not os.path.exists(po_path):
            print "Error: Could not find the directory %s" % lang_path
            sys.exit(1)
        if not os.path.exists(lc_path):
            os.mkdir(lc_path)
        
        oldsys = sys.argv
        sys.argv = [sys.argv[0], '-o', mo_path, po_path]

        import pylons.i18n.msgfmt
        pylons.i18n.msgfmt.main()
        print "Sucessfully generated '%s' catalog" % lang
        
        sys.argv = oldsys
        
    def run(self):
        project = re.compile('[^a-zA-Z0-9_]').sub('', self.distribution.get_name().lower())
        if not self.lang:
            print "No langauge specified, compiling all languages"
            i18n_path = os.path.join(project, 'i18n')
            for lang in os.listdir(i18n_path):
                if os.path.isdir(os.path.join(i18n_path, lang)) and \
                                     lang not in exclude_dirs:
                    self.compile_lang(lang)
        else:
            self.compile_lang(self.lang)



"""Base objects to be exported for use in Controllers"""
from paste.registry import StackedObjectProxy
from paste.wsgiwrappers import WSGIResponse as Response

from pylons.controllers import Controller, RPCController
from pylons.decorators import jsonify
from pylons.legacy import m, params

c = StackedObjectProxy(name="C")
g = StackedObjectProxy(name="G")
cache = StackedObjectProxy(name="Cache")
session = StackedObjectProxy(name="Session")
request = StackedObjectProxy(name="Request")
buffet = StackedObjectProxy(name="Buffet")
h = StackedObjectProxy(name="h")

translator = StackedObjectProxy(name="Translator")

__all__ = ['Controller', 'RPCController', 'Response', 'jsonify']



"""Paster Commands, for use with paster in your project

The command(s) listed here are for use with Paste to enable easy creation of
various core Pylons templates.

Currently available commands are::

    controller, shell
"""
import os
import sys
import pylons
import pylons.util as util

from paste.script.command import Command, BadCommand
from paste.script.filemaker import FileOp
from paste.deploy import loadapp, appconfig
import paste.deploy.config
import paste.fixture

def validate_name(name):
    """Validate that the name for the controller isn't present on the
    path already"""
    if not name:
        # This happens when the name is an existing directory
        raise BadCommand('Please give the name of a controller.')
    try:
        __import__(name)
        raise BadCommand(
            "\n\nA module named '%s' is already present in your "
            "PYTHON_PATH.\n Choosing a conflicting name will likely cause"
            " import problems in\n your controller at some point. It's "
            "suggested that you choose an alternate\nname, and if you'd "
            "like that name to be accessible as '%s', add a route\n"
            "to your projects config/routing.py file similar to:\n"
            "  map.connect('%s', controller='my_%s')" \
            % (name, name, name, name))
    except ImportError:
        # This is actually the result we want
        pass
    
    return True

class ControllerCommand(Command):
    """Create a Controller and functional test for it
    
    The Controller command will create the standard controller template
    file and associated functional test to speed creation of controllers.
    
    Example usage::
    
        yourproj% paster controller comments
        Creating yourproj/yourproj/controllers/comments.py
        Creating yourproj/yourproj/tests/functional/test_comments.py
    
    If you'd like to have controllers underneath a directory, just include
    the path as the controller name and the necessary directories will be
    created for you::
    
        yourproj% paster controller admin/trackback
        Creating yourproj/controllers/admin
        Creating yourproj/yourproj/controllers/admin/trackback.py
        Creating yourproj/yourproj/tests/functional/test_admin_trackback.py
    """
    summary = __doc__
    usage = 'CONTROLLER_NAME'
    
    min_args = 1
    max_args = 1
    group_name = 'pylons'
    
    default_verbosity = 3
    
    parser = Command.standard_parser(simulate=True)
    parser.add_option('--no-test',
                      action='store_true',
                      dest='no_test',
                      help="Don't create the test; just the controller")
            
    def command(self):
        """Main command to create controller"""
        try:
            file_op = FileOp(source_dir=os.path.join(
                os.path.dirname(__file__), 'templates'))
            try:
                name, directory = file_op.parse_path_name_args(self.args[0])
            except:
                raise BadCommand('No egg_info directory was found')
            
            # Validate the name
            name = name.replace('-', '_')
            validate_name(name)
            
            fullname = os.path.join(directory, name)
            controller_name = util.class_name_from_module_name(
                name.split('/')[-1])
            if not fullname.startswith(os.sep):
                fullname = os.sep + fullname
            testname = fullname.replace(os.sep, '_')[1:]
            file_op.template_vars.update({'name': controller_name,
                                  'fname': os.path.join(directory, name)})
            file_op.copy_file(template='controller.py_tmpl',
                         dest=os.path.join('controllers', directory), 
                         filename=name)
            if not self.options.no_test:
                file_op.copy_file(template='test_controller.py_tmpl',
                             dest=os.path.join('tests', 'functional'),
                             filename='test_'+testname)
        except:
            msg = str(sys.exc_info()[1])
            raise BadCommand('An unknown error ocurred, %s' % msg)

class ShellCommand(Command):
    """Open an interactive shell with the Pylons app loaded
    
    The optional CONFIG_FILE argument specifies the config file to use for
    the interactive shell. CONFIG_FILE defaults to 'development.ini'.
    
    This allows you to test your mapper, models, and simulate web requests
    using ``paste.fixture``.
    
    Example::
        
        $ paster shell my-development.ini
    """
    summary = __doc__
    usage = '[CONFIG_FILE]'
    
    min_args = 0
    max_args = 1
    group_name = 'pylons'
    
    parser = Command.standard_parser(simulate=True)

    def command(self):
        """Main command to create a new shell"""
        self.verbose = 3
        
        if len(self.args) == 0:
            # Assume the .ini file is ./development.ini
            config_file = 'development.ini'
            if not os.path.isfile(config_file):
                raise BadCommand('%sError: CONFIG_FILE not found at: .%s%s\n'
                                 'Please specify a CONFIG_FILE' % \
                                 (self.parser.get_usage(), os.path.sep,
                                  config_file))
        else:
            config_file = self.args[0]
            
        config_name = 'config:%s' % config_file
        here_dir = os.getcwd()
        locs = dict(__name__="pylons-admin")
        pkg_name = here_dir.split(os.path.sep)[-1].lower()
        
        # Load app config into paste.deploy to simulate request config
        app_conf = appconfig(config_name, relative_to=here_dir)
        conf = dict(app=app_conf, app_conf=app_conf)
        paste.deploy.config.CONFIG.push_thread_config(conf)
        
        # Load locals and populate with objects for use in shell
        sys.path.insert(0, here_dir)
        
        # Load the wsgi app first so that everything is initialized right
        wsgiapp = loadapp(config_name, relative_to=here_dir)
        
        # Start the rest of our imports now that the app is loaded
        routing_package = pkg_name + '.config.routing'
        models_package = pkg_name + '.models'
        helpers_package = pkg_name + '.lib.helpers'
        
        # Import all the modules
        for pack in [routing_package, models_package, helpers_package]:
            __import__(pack)
        
        make_map = getattr(sys.modules[routing_package], 'make_map')
        mapper = make_map()
        test_app = app=paste.fixture.TestApp(wsgiapp)
        global_obj = None
        try:
            global_obj = test_app.get('/').g
        except:
            pass
        
        locs.update(
            dict(
                model=sys.modules[models_package],
                mapper=mapper,
                wsgiapp=wsgiapp,
                app=test_app,
                h=sys.modules[helpers_package],
            )
        )
        if global_obj:
            locs['g'] = global_obj
            pylons.g.push_object(global_obj)
        
        banner = "Pylons Interactive Shell\nPython %s\n\n" % sys.version
        banner += "Additional Objects:\n"
        banner += "  %-10s -  %s\n" % ('mapper', 'Routes mapper object')
        banner += "  %-10s -  %s\n" % ('h', 'Helper object')
        if global_obj:
            banner += "  %-10s -  %s\n" % ('g', 'Globals object')
        banner += "  %-10s -  %s\n" % ('model', 'Models from models package')
        banner += "  %-10s -  %s\n" % ('wsgiapp', 
            'This projects WSGI App instance')
        banner += "  %-10s -  %s\n" % ('app', 
            'paste.fixture wrapped around wsgiapp')
        try:
            # try to use IPython if possible
            import IPython
        
            class CustomIPShell(IPython.iplib.InteractiveShell):
                """Custom shell class to handle raw input"""
                def raw_input(self, *args, **kw):
                    """Capture raw input in exception wrapping"""
                    try:
                        return IPython.iplib.InteractiveShell.raw_input(
                            self, *args, **kw)
                    except EOFError:
                        # In the future, we'll put our own override as needed 
                        # to save models, TG style
                        raise EOFError

            shell = IPython.Shell.IPShell(user_ns=locs, 
                shell_class=CustomIPShell)
            shell.mainloop()
        except ImportError:
            import code
            
            class CustomShell(code.InteractiveConsole):
                """Custom shell class to handle raw input"""
                def raw_input(self, *args, **kw):
                    """Capture raw input in exception wrapping"""
                    try:
                        return code.InteractiveConsole.raw_input(
                            self, *args, **kw)
                    except EOFError:
                        # In the future, we'll put our own override as needed 
                        # to save models, TG style
                        raise EOFError
            
            shell = CustomShell(locals=locs)
            try:
                import readline
            except ImportError:
                pass
            shell.interact(banner)



from projectname.lib.base import *
from pylons.decorators import rest

class SampleController(BaseController):
    def index(self):
        return Response('basic index page')
    
    def session_increment(self):
        session.setdefault('counter', -1)
        session['counter'] += 1
        session.save()
        return Response('session incrementer')
    
    def globalup(self):
        return Response(g.message)
    
    def global_store(self, id):
        if id:
            g.counter += int(id)
        return Response(str(g.counter))
    
    def myself(self):
        return Response(h.url_for())
    
    def myparams(self):
        return Response(str(request.params))
    
    def testdefault(self):
        return render_response('testkid')

    def test_extra_engine(self):
        return render_response('kid', 'testkid')
    
    def test_template_caching(self):
        return render_response('/test_myghty.myt', cache_expire='never')
    
    def test_only_post(self):
        return Response('It was a post!')
    test_only_post = rest.dispatch_on(GET='test_only_get')(rest.restrict('POST')(test_only_post))
    
    def test_only_get(self):
        return Response('It was a get!')
    test_only_get = rest.restrict('GET')(test_only_get)
    
    def impossible(self):
        return Response('This should never be shown')
    impossible = rest.restrict('POST')(rest.dispatch_on(POST='test_only_post')(impossible))

    def testcheetah(self):
        c.test = "This is in c var"
        return render_response('testcheetah')



"""Helpers object, Paste Template config and misc. functionality.

The util module provides the main Helper object used by Pylons.

PylonsTemplate is a Paste Template sub-class that configures the source
directory and default plug-ins for a new Pylons project.
"""
import os.path
from paste.deploy.config import CONFIG
from paste.script.templates import Template
import pylons

def log(msg):
    """Log a message to the output log."""
    pylons.request.environ['wsgi.errors'].write('=> %s\n'%str(msg))

def _(value):
    """Mark a string for translation
    
    Mark a string to be internationalized as follows:
    
    .. code-block:: Python
    
        h._('This should be in lots of languages')
    """
    return pylons.translator['translator'].gettext(value)

def set_lang(lang):
    """Set the language used"""
    if lang is None:
        pylons.translator['translator'] = _Translator()
    else:
        from pkg_resources import resource_exists
        from pylons.i18n.translation import egg_translation
        project_name = CONFIG['app_conf']['package']
        catalog_path = os.path.join('i18n', lang, 'LC_MESSAGES')
        if not resource_exists(project_name, catalog_path):
            raise LanguageError('Language catalog %s not found' % \
                                os.path.join(project_name, catalog_path))
        pylons.translator['translator'] = \
            egg_translation(project_name, lang=catalog_path)

def get_lang():
    return pylons.translator.get('lang')

def get_prefix(environ):
    if 'paste.config' in environ:
        prefix = environ['paste.config']['app_conf'].get('prefix', '')
    else:
        # Not ideal but if the error occurs before the paste.config is available not a lot we can do
        prefix = ''
    if not prefix:
        if environ.get('SCRIPT_NAME', '') != '':
            prefix = environ['SCRIPT_NAME']
    return prefix

def class_name_from_module_name(module_name):
    """Takes a module name and returns the name of the class it defines.

    If the module name contains dashes, they are replaced with underscores.
    
    Example::
    
        >>> class_name_from_module_name('with-dashes')
        'WithDashes'
        >>> class_name_from_module_name('with_underscores')
        'WithUnderscores'
        >>> class_name_from_module_name('oneword')
        'Oneword'
    """
    words = module_name.replace('-', '_').split('_')
    return ''.join([w.title() for w in words])

class ContextObj(object):
    """ The 'c' object, with strict attribute access (raises an Exception when
    the attribute does not exist) """
    pass

class AttribSafeContextObj(object):
    """ The 'c' object, with lax attribute access (returns '' when the attribute
    does not exist) """
    def __getattr__(self, name):
        try:
            return object.__getattribute__(self, name)
        except AttributeError:
            return ''

class LanguageError(Exception):
    """Exception raised when a problem occurs with changing languages"""
    pass

class _Translator(object):
    """An empty gettext translator which just returns the original string"""
    def gettext(self, value):
        return value

class PylonsTemplate(Template):
    _template_dir = 'templates/default_project'
    summary = 'Pylons application template'
    egg_plugins = ['Pylons', 'WebHelpers']

class MinimalPylonsTemplate(Template):
    _template_dir = 'templates/minimal_project'
    summary = 'Pylons minimal application template'
    egg_plugins = ['Pylons', 'WebHelpers']
    

__all__ = ['AttribSafeContextObj', 'ContextObj', 'Helpers', 'class_name_from_module_name',
    'log', '_', 'set_lang', 'get_lang']
__pudge_all__ = __all__ + ['PylonsTemplate']



"""``etag_cache``, ``redirect_to``, and ``abort`` methods

Additional helper object available for use in Controllers is the etag_cache.
"""
from routes import redirect_to
import paste.httpexceptions as httpexceptions

import pylons

def etag_cache(key=None):
    """Use the HTTP Entity Tag cache for Browser side caching
    
    If a "If-None-Match" header is found, and equivilant to ``key``, then
    a ``304`` HTTP message will be returned with the ETag to tell the browser
    that it should use its current cache of the page.
    
    Otherwise, the ETag header will be added to a new response object and
    returned for use in your action.
    
    Suggested use is within a Controller Action like so:
    
    .. code-block:: Python
    
        import pylons
        
        class YourController(BaseController):
            def index(self):
                resp = etag_cache(key=1)
                resp.write(render('/splash.myt'))
                return resp
    
    .. Note:: 
        This works because etag_cache will raise an HTTPNotModified
        exception if the ETag recieved matches the key provided.
    """
    if_none_match = pylons.request.environ.get('HTTP_IF_NONE_MATCH', None)
    resp = pylons.Response()
    resp.headers['ETag'] = key
    if str(key) == if_none_match:
        raise httpexceptions.HTTPNotModified()
    else:
        return resp

def abort(status_code=None, detail="", headers=None, comment=None):
    """Aborts the request immediately by returning an HTTP exception
    
    In the event that the status_code is a 300 series error, the detail 
    attribute will be used as the Location header should one not be specified
    in the headers attribute.
    """
    exc = httpexceptions.get_exception(status_code)(detail, headers, comment)
    raise exc

__all__ = ['etag_cache', 'redirect_to', 'abort']



"""Standard Controllers intended for sub-classing by web developers"""
import types
import inspect
import xmlrpclib

from paste.httpexceptions import HTTPException
from paste.deploy.config import CONFIG
from paste.deploy.converters import asbool

import pylons

class Controller(object):
    """Standard Pylons Controller for Web Requests
    
    The Pylons Controller handles incoming web requests that are dispatched
    from the custom Myghty Routes resolver. These requests result in a new
    instance of the Controller being created, which is then called with the
    dict options from the Routes match.
    
    By default, the Controller will search and attempt to call a ``__before__``
    method before calling the action, and will try to call a ``__after__``
    method after the action was called. These two methods can act as filters
    controlling access to the action, setup variables/objects for use with a
    set of actions, etc.
    
    Each action to be called is inspected with ``_inspect_call`` so that it is
    only passed the arguments in the Routes match dict that it asks for. The only
    exception to the dict is that the Myghty ``ARGS`` variable is included.
    
    In the event that an action is not found to handle the request, the Controller
    will raise an "Action Not Found" error if in debug mode, otherwise a ``404 Not Found``
    error will be returned.
    """
    __pudge_all__ = ['_inspect_call', '__call__', '_attach_locals']
    
    def _attach_locals(self):
        """Attach Pylons special objects to the controller
        
        When debugging, the Pylons special objects are unavailable because they
        are thread locals. This function pulls the actual object and attaches it
        to the controller so that it can be examined for debugging purposes.
        """
        self.c = pylons.c._current_obj()
        self.g = pylons.g._current_obj()
        self.cache = pylons.cache._current_obj()
        self.session = pylons.session._current_obj()
        self.request = pylons.request._current_obj()
        self.buffet = pylons.buffet._current_obj()
        self.start_response = None
    
    def _inspect_call(self, func, **kargs):
        """Calls a function with the Routes dict
        
        Given a function, inspect_call will inspect the function args and call
        it with no further keyword args than it asked for.
        
        If the function has been decorated, it is assumed that the decorator
        preserved the function signature.
        """
        argspec = inspect.getargspec(func)
        req = pylons.request._current_obj()
        kargs = req.environ['pylons.routes_dict'].copy()
        kargs.update(dict(environ=req.environ, 
            start_response=self.start_response))
        
        # @@ LEGACY: Add in ARGS alias to params
        if 'pylons.legacy' in req.environ:
            kargs['ARGS'] = req._legacy_params
        
        # Hide the traceback for everything above this controller
        __traceback_hide__ = 'before_and_this'
        
        c = pylons.c._current_obj()
        if argspec[2]:
            for k, val in kargs.iteritems():
                setattr(c, k, val)
            return func(**kargs)
        else:
            argnames = argspec[0][1:]
            args = []
            for name in argnames:
                if name in kargs:
                    setattr(c, name, kargs[name])
                    args.append(kargs[name])
            return func(*args)
    
    def _dispatch_call(self):
        """Handles dispatching the request to the function"""
        req = pylons.request._current_obj()
        action = req.environ['pylons.routes_dict'].get('action')
        action_method = action.replace('-', '_')
        func = getattr(self, action_method, None)
        if isinstance(func, types.MethodType):
            response = self._inspect_call(func)
        else:
            if asbool(CONFIG['global_conf'].get('debug')):
                raise NotImplementedError(
                    'Action %s is not implemented' % action)
            else:
                response = pylons.Response(code=404)
        return response
    
    def __call__(self, *args, **kargs):
        """Makes our controller a callable to handle requests
        
        This is called when dispatched to as the Controller class docs explain
        more fully.
        """
        req = pylons.request._current_obj()
        
        # Keep private methods private
        if req.environ['pylons.routes_dict'].get('action', '').startswith('_'):
            return pylons.Response(code=404)
        
        if hasattr(self, '__before__'):
            self._inspect_call(self.__before__, **kargs)

        if not hasattr(self, '__after__'):
            response = self._dispatch_call()
        else:
            try:
                response = self._dispatch_call()
            except HTTPException:
                self._inspect_call(self.__after__)
                raise
            self._inspect_call(self.__after__)
        return response

class WSGIController(Controller):
    """WSGI Controller that follows WSGI spec for calling and return values
    
    This function works identically to the normal Controller, however it is called
    with the WSGI interface, and behaves as a WSGI application calling start_response
    and returning an iterable as content.
    """
    def __call__(self, environ, start_response):
        self.start_response = start_response
        
        # Keep private methods private
        if environ['pylons.routes_dict'].get('action', '').startswith('_'):
            return pylons.Response(code=404)
        
        if hasattr(self, '__before__'):
            self._inspect_call(self.__before__)

        if not hasattr(self, '__after__'):
            response = self._dispatch_call()
        else:
            try:
                response = self._dispatch_call()
            except HTTPException:
                self._inspect_call(self.__after__)
                raise
            self._inspect_call(self.__after__)

        if hasattr(response, 'wsgi_response'):
            # Pull the content we need for a WSGI response
            status, response_headers, content = response.wsgi_response()
            start_response(status, response_headers)
        
            # Copy the response object into the testing vars if we're testing
            if 'paste.testing' in environ:
                environ['paste.testing_variables']['response'] = response
            response = content
        
        return response
        

class RPCController(Controller):
    resource = 'RPC2'

    def __call__(self, environ, start_response):
        self.start_response = start_response
        match = environ['pylons.routes_dict']
        
        # Keep private methods private
        if match.get('action', '').startswith('_'):
            return pylons.Response(code=404)

        if  match.get('action') != RPCController.resource:
            if environ['paste.config']['global_conf']['debug'] == 'false':
                return pylons.Response(code=404)
            else:
                raise NotImplementedError(
                    'RPCController only supports %s action', 
                    RPCController.resource)
        
        if hasattr(self, '__before__'):
            self._inspect_call(self.__before__)
        response = pylons.Response(
            xmlrpclib.dumps(
                (self._dispatch_call().wsgi_response()[2],)
            )
        )
        if hasattr(self, '__after__'):
            self._inspect_call(self.__after__)
        return response
    
    def __call__2(self, action, **kargs):
        req = pylons.request._current_obj()
        action = req.environ['pylons.routes_dict'].get('action')
        action_method = action.replace('-', '_')
        if action_method != RPCController.resource:
            if asbool(CONFIG['global_conf'].get('debug')):
                raise NotImplementedError(
                    'RPCController only supports %s action',
                    RPCController.resource)
            else:
                return pylons.Response(code=404)
        d = req.environ['wsgi.input'].read()
        params, method = xmlrpclib.loads(d)
        
        if hasattr(self, '__before__'):
            self.__before__(method, **params)
        if isinstance(getattr(self, method, None), types.MethodType):
            self.__getattribute__(method)(**params)
        else:
            res = 3#supposed to return xmlrpc fault thing
        response = pylons.Response(xmlrpclib.dumps((res,)))
        if hasattr(self, '__after__'):
            self.__after__(pylons, method, **params)
        return response

__all__ = ['Controller', 'WSGIController', 'RPCController']



"""WSGI App Creator

This module is responsible for creating the basic Pylons WSGI application. It's generally
assumed that it will be called by Paste, though any WSGI application server could create
and call the WSGI app as well.
"""
import sys
import re
import inspect
import urllib
import warnings

import paste.wsgiwrappers
import paste.httpexceptions as httpexceptions
from paste.registry import RegistryManager
from paste.wsgiwrappers import WSGIRequest

from routes import request_config

import pylons
from pylons.util import ContextObj, AttribSafeContextObj, _Translator, set_lang, class_name_from_module_name
from pylons.controllers import Controller, WSGIController

class PylonsBaseWSGIApp(object):
    """Basic Pylons WSGI Application

    This basic WSGI app is provided should a web developer want to
    get access to the most basic Pylons web application environment
    available. By itself, this Pylons web application does little more
    than dispatch to a controller and setup the context object, the
    request object, and the globals object.
    
    Additional functionality like sessions, and caching can be setup by
    altering the ``environ['pylons.environ_config']`` setting to indicate
    what key the ``session`` and ``cache`` functionality should come from.
    
    Resolving the URL and dispatching can be customized by sub-classing or
    "monkey-patching" this class. Subclassing is the preferred approach.
    """
    def __init__(self, mapper, package_name, globals, 
                 helpers=None, default_charset=None):
        """Initialize a base Pylons WSGI application
        
        The base Pylons WSGI application requires several keywords, if no 
        helpers reference is given, a legacy warning will be raised and it
        will attempt to search the project for a helpers.py module.
        """
        self.mapper = mapper
        self.helpers = helpers
        self.globals = globals
        self.package_name = package_name
        config = globals.pylons_config
        if default_charset:
            warnings.warn(
                "The 'default_charset' keyword argument to the PylonsBaseWSGIApp "
                "constructor is deprecated. Please specify 'default_charset' to the Config "
                'object in your config/environment.py file instead, e.g.:\n'
                'return pylons.config.Config(myghty, map, paths, '
                "default_charset='%s')" % default_charset, DeprecationWarning, 2)
            config.default_charset = default_charset
        self.settings = dict(content_type='text/html', charset=config.default_charset)
        
        # Create the redirect function we'll use and save it
        def redirect_to(url):
            raise httpexceptions.HTTPFound(url)
        self.redirect_to = redirect_to
        
        # Initialize Buffet and all our template engines, default engine is the
        # first in the template_engines list
        def_eng = config.template_engines.pop(0)
        self.buffet = pylons.templating.Buffet(def_eng['engine'], 
            template_root=def_eng['template_root'], **def_eng['template_options'])
        for e in config.template_engines:
            self.buffet.prepare(e['engine'], template_root=e['template_root'], 
                alias=e['alias'], **e['template_options'])
    
    def __call__(self, environ, start_response):
        self.setup_app_env(environ, start_response)
        if environ.get('paste.testing'):
            self.load_test_env(environ)
        
        # Change our HTTP_METHOD if _method is present, try GET first to avoid
        # parsing POST unless absolutely necessary.
        req = pylons.request._current_obj()
        old_method = None
        if '_method' in environ.get('QUERY_STRING', '') and '_method' in req.GET:
            old_method = environ['REQUEST_METHOD']
            environ['REQUEST_METHOD'] = req.GET['_method']
        elif environ['REQUEST_METHOD'] == 'POST' and '_method' in req.POST:
            old_method = environ['REQUEST_METHOD']
            environ['REQUEST_METHOD'] = req.POST['_method']
        
        controller = self.resolve(environ, start_response)
        if old_method: environ['REQUEST_METHOD'] = old_method            
        response = self.dispatch(controller, environ, start_response)
        
        if environ.get('paste.testing') and hasattr(response, 'wsgi_response'):
            environ['paste.testing_variables']['response'] = response
        
        # Transform HttpResponse objects into WSGI response
        if hasattr(response, 'wsgi_response'):
            status, response_headers, content = response.wsgi_response()
            start_response(status, response_headers)
            return content
        elif response:
            return response
            
        # Apparently we returned absolutely nothing, use the response
        # object if in legacy mode, otherwise raise an exception
        if environ.get('pylons.legacy'):
            resp = pylons.legacy.response
            if hasattr(pylons.legacy.response, 'wsgicall'):
                # Legacy app using run_wsgi
                warnings.warn(
                    "Using run_wsgi is deprecated. You should use return "
                    "wsgiapp(request.environ, self.start_response) instead. ",
                    DeprecationWarning, 2)
                return resp.content
            warnings.warn(
                "Running in legacy mode, ensure that your controller actions "
                "are returning a Pylons Response and not using the 'm' object "
                "as it has been deprecated.",
                DeprecationWarning, 2)
            status, response_headers, content = resp.wsgi_response()
            start_response(status, response_headers)
            return content
        else:
            raise Exception, "No content returned by controller: %s" % controller.__name__
    
    def setup_app_env(self, environ, start_response):
        """Setup and register all the Pylons objects with the registry"""
        req = WSGIRequest(environ)
        
        helpers = self.helpers or pylons.legacy.load_h(self.package_name)
        environ['paste.registry'].register(pylons.h, helpers)
        
        # Setup the translator global object
        trans = dict(translator=_Translator())
        trans['CONFIG'] = dict(app_conf=self.globals.pylons_config.app_conf,
            global_conf=self.globals.pylons_config.global_conf)
        environ['paste.registry'].register(pylons.translator, trans)
        set_lang(self.globals.pylons_config.app_conf.get('lang'))
        
        # Setup the basic pylons global objects
        environ['paste.registry'].register(paste.wsgiwrappers.settings, self.settings)
        environ['paste.registry'].register(pylons.request, req)
        if self.globals.pylons_config.strict_c:
            environ['paste.registry'].register(pylons.c, ContextObj())
        else:
            environ['paste.registry'].register(pylons.c, AttribSafeContextObj())
        environ['paste.registry'].register(pylons.g, self.globals)
        environ['paste.registry'].register(pylons.buffet, self.buffet)
        
        # Setup legacy globals
        if environ.get('pylons.legacy'):
            # Legacy mixed dictionary instead of MultiDict
            req._legacy_params = req.params.mixed()
            environ['paste.registry'].register(pylons.legacy.response, WSGIResponse())
            environ['paste.registry'].register(pylons.m, 
                pylons.legacy.MyghtyCompat(environ, start_response))
            environ['paste.registry'].register(pylons.params, req._legacy_params)
        
        econf = environ['pylons.environ_config']
        if econf.get('session'):
            environ['paste.registry'].register(pylons.session, environ[econf['session']])
        if econf.get('cache'):
            environ['paste.registry'].register(pylons.cache, environ[econf['cache']])
    
    def resolve(self, environ, start_response):
        """Implements Routes-based dispatching"""
        config = request_config()
        config.mapper = self.mapper
        config.environ = environ
        config.redirect = self.redirect_to
        match = config.mapper_dict
        if not match:
            return None
        environ['pylons.routes_dict'] = match
        controller = match.get('controller')
        if not controller:
            return None
        
        # Pull the controllers class name, import controller
        # @@ TODO: Encapsulate in try/except to raise error if controller
        #          doesn't exist, or class in controller file doesn't exist.
        full_module_name = self.package_name + '.controllers.' \
            + controller.replace('/', '.')
        
        # Hide the traceback here if the import fails (bad syntax and such)
        __traceback_hide__ = 'before_and_this'
        
        __import__(full_module_name)
        module_name = controller.split('/')[-1]
        class_name = class_name_from_module_name(module_name) + 'Controller'
        return getattr(sys.modules[full_module_name], class_name)
        
    def dispatch(self, controller, environ, start_response):
        """Dispatches to a controller, will instantiate the controller if
        necessary"""
        if not controller:
            raise httpexceptions.HTTPNotFound()
        match = environ['pylons.routes_dict']
        
        # Sanitize keys
        # @@ TODO: This should be done in a lazy fashion
        for k,v in match.iteritems():
            if v:
                match[k] = urllib.unquote_plus(v)
        
        # Older subclass of Controller
        if inspect.isclass(controller) and not issubclass(controller, WSGIController) and \
                issubclass(controller, Controller):
            controller = controller()
            controller.start_response = start_response
            
            # @@ LEGACY: Attach c to controller
            if environ.get('pylons.legacy'):
                controller.c = pylons.c
            
            return controller(**match)
        
        # If the route included a path_info attribute, we'll assume it
        # should be pulled, otherwise we call the controller    
        if match.get('path_info'):
            self.fixup_environ(environ, match)
        
        # If it's a class, instantiate it
        if not hasattr(controller, '__class__') or \
            getattr(controller, '__class__') == type:
            controller = controller()
        
        # Controller is assumed to handle a WSGI call
        return controller(environ, start_response)
    
    def fixup_environ(self, environ, dispatch):
        """Fixes the environ based on the Routes match"""
        oldpath = environ['PATH_INFO']
        newpath = dispatch.get('path_info') or ''
        environ['PATH_INFO'] = newpath
        if not environ['PATH_INFO'].startswith('/'):
            environ['PATH_INFO'] = '/' + environ['PATH_INFO']
        environ['SCRIPT_NAME'] += re.sub(r'^(.*?)/' + newpath + '$', r'\1', oldpath)
        if environ['SCRIPT_NAME'].endswith('/'):
            environ['SCRIPT_NAME'] = environ['SCRIPT_NAME'][:-1]
    
    def load_test_env(self, environ):
        """Sets up our Paste testing environment"""
        testenv = environ['paste.testing_variables']
        testenv['req'] = pylons.request._current_obj()
        testenv['c'] = pylons.c._current_obj()
        testenv['g'] = pylons.g._current_obj()
        econf = environ['pylons.environ_config']
        if econf.get('session'):
            testenv['session'] = environ[econf['session']]
        if econf.get('cache'):
            testenv['cache'] = environ[econf['cache']]
        
    
class PylonsApp(object):
    """Setup the Pylons default environment
    
    Pylons App sets up the basic Pylons app, and initializes the global
    object, sessions and caching. Sessions and caching can be overridden
    in the config object by supplying other keys to look for in the environ
    where objects for the session/cache will be. If they're set to none,
    then no session/cache objects will be available.
    """
    def __init__(self, config, default_charset=None, helpers=None, g=None):
        self.config = config
        if default_charset:
            warnings.warn(
                "The 'default_charset' keyword argument to the PylonsApp constructor is "
                "deprecated. Please specify 'default_charset' to the Config object in your "
                'config/environment.py file instead, e.g.:\n'
                'return pylons.config.Config(myghty, map, paths, '
                "default_charset='%s')" % default_charset, DeprecationWarning, 2)
            self.config.default_charset = default_charset

        if not g:
            warnings.warn(
                "Having the 'g' object load from a default app_globals module "
                "is deprecated. Please update your middleware.py with:\n\n"
                "    import MYPROJ.lib.app_globals as app_globals\n"
                "    import MYPROJ.lib.helpers\n\n"
                "where MYPROJ is the name of your project.\n"
                "Then edit the PylonsApp instantiation with:\n\n"
                "    app = pylons.wsgiapp.PylonsApp(\n"
                "        config, \n"
                "        helpers=MYPROJ.lib.helpers, \n"
                "        g=app_globals.Globals\n"
                "    )\n\n",
                DeprecationWarning, 2)
            try:
                globals_package = __import__(config.package + '.lib.app_globals', globals(), locals(), ['Globals'])
                g = getattr(globals_package, 'Globals')
            except ImportError:
                pass
        if not g:
            # Assign a default globals object
            g = type("Globals", (), {})
        if g:
            g = g(config.global_conf, config.app_conf, config=config)
            g.pylons_config = config
        
        # Create the base Pylons wsgi app
        app = PylonsBaseWSGIApp(config.map, config.package, g, helpers=helpers)
        
        # Pull user-specified environ overrides, or just setup default
        # session and caching objects
        self.econf = econf = config.environ_config.copy()
        if 'session' not in econf:
            from beaker.session import SessionMiddleware
            econf['session'] = 'beaker.session'
            app = SessionMiddleware(app, config.global_conf, 
                auto_register=True, **config.app_conf)
        
        if 'cache' not in econf:
            from beaker.cache import CacheMiddleware
            econf['cache'] = 'beaker.cache'
            app = CacheMiddleware(app, config.global_conf, **config.app_conf)
        
        self.globals = g
        self.app = app
    
    def __call__(self, environ, start_response):
        environ['pylons.environ_config'] = self.econf
        return self.app(environ, start_response)

WSGIResponse = None

class LegacyApp(object):
    def __init__(self, config):
        global WSGIResponse
        from paste.wsgiwrappers import WSGIResponse as WResponse
        WSGIResponse = WResponse
        self.app = PylonsApp(config)
        self.globals = self.app.globals
    
    def __call__(self, environ, start_response):
        environ['pylons.legacy'] = True
        return self.app(environ, start_response)

def make_app(config):
    """ Legacy WSGI app creator"""
    warnings.warn(
        "Legacy WSGI app in use for pre-0.9 application. This will be "
        "removed before the release of 1.0 which will require minor updates "
        "to your application.",
        DeprecationWarning, 2)
    papp = LegacyApp(config)
    from paste.deploy.config import ConfigMiddleware
    app = ConfigMiddleware(papp, {
        'default':config.global_conf,
        'app':config.app_conf,
        'app_conf':config.app_conf,
        'global_conf':config.global_conf
    })
    app = RegistryManager(app)
    app.globals = papp.globals
    return app



"""Decorator module

Author:         Michele Simionato
E-mail:         michele.simionato@gmail.com
Version:        0.7
License:        Python license

For more details, see:
http://www.phyast.pitt.edu/~micheles/python/documentation.html
"""

__all__ = ["decorator", "newfunc", "getinfo"]

import inspect
import itertools
import new
import sys

def getinfo(func):
    """Return an info dictionary containing:
    - name (the name of the function : str)
    - argnames (the names of the arguments : list)
    - defarg (the values of the default arguments : list)
    - fullsign (the full signature : str)
    - shortsign (the short signature : str)
    - arg0 ... argn (shortcuts for the names of the arguments)

    >>> def f(self, x=1, y=2, *args, **kw): pass

    >>> info = getinfo(f)

    >>> info["name"]
    'f'
    >>> info["argnames"]
    ['self', 'x', 'y', 'args', 'kw']
    
    >>> info["defarg"]
    (1, 2)

    >>> info["shortsign"]
    'self, x, y, *args, **kw'
    
    >>> info["fullsign"]
    'self, x=defarg[0], y=defarg[1], *args, **kw'

    >>> info["arg0"], info["arg1"], info["arg2"], info["arg3"], info["arg4"]
    ('self', 'x', 'y', 'args', 'kw')
    """
    assert inspect.ismethod(func) or inspect.isfunction(func)
    regargs, varargs, varkwargs, defaults = inspect.getargspec(func)
    argnames = list(regargs)
    if varargs: argnames.append(varargs)
    if varkwargs: argnames.append(varkwargs)
    counter = itertools.count()
    fullsign = inspect.formatargspec(
        regargs, varargs, varkwargs, defaults,
        formatvalue=lambda value: "=defarg[%i]" % counter.next())[1:-1]
    shortsign = inspect.formatargspec(
        regargs, varargs, varkwargs, defaults,
        formatvalue=lambda value: "")[1:-1]
    dic = dict([("arg%s" % n, name) for n, name in enumerate(argnames)])
    dic.update(name=func.__name__, argnames=argnames, shortsign=shortsign,
        fullsign = fullsign, defarg = func.func_defaults or ())
    return dic

def _contains_reserved_names(dic): # helper
    return "_call_" in dic or "_func_" in dic

def _decorate(func, caller, infodict=None):
    """Takes a function and a caller and returns the function
    decorated with that caller. The decorated function is obtained
    by evaluating a function with the correct signature.
    """
    infodict = infodict or getinfo(func)
    assert not _contains_reserved_names(infodict["argnames"]), \
           "You cannot use _call_ or _func_ as argument names!"
    execdict= dict(_func_=func, _call_=caller, defarg=infodict["defarg"])
    if infodict['name'] == "<lambda>":
        lambda_src = "lambda %(fullsign)s: _call_(_func_, %(shortsign)s)" \
                     % infodict
        dec_func = eval(lambda_src, execdict)
    else:
        func_src = """def %(name)s(%(fullsign)s):
        return _call_(_func_, %(shortsign)s)""" % infodict
        # import sys; print >> sys.stderr, func_src # for debugging 
        exec func_src in execdict 
        dec_func = execdict[infodict['name']]
    dec_func.__doc__ = func.__doc__
    dec_func.__dict__ = func.__dict__
    dec_func.__module__ = func.__module__
    return dec_func

class decorator(object):
    """General purpose decorator factory: takes a caller function as
input and returns a decorator. A caller function is any function like this::

    def caller(func, *args, **kw):
        # do something
        return func(*args, **kw)
    
Here is an example of usage:

    >>> @decorator
    ... def chatty(f, *args, **kw):
    ...     print "Calling %r" % f.__name__
    ...     return f(*args, **kw)
    
    >>> @chatty
    ... def f(): pass
    ...
    >>> f()
    Calling 'f'
    """
    def __init__(self, caller):
        self.caller = caller
    def __call__(self, func):
        if sys.version < '2.4': # gracefull fallback
            return lambda *args, **kw : self.caller(func, *args, **kw)
        else:
            return _decorate(func, self.caller)

def newfunc(func, model=None): # not used internally
    """Creates an independent copy of a function. If model is not None,
    the new function copies the signature, as well the attributes
    __name__, __doc__, __dict__ and __module__ from the model, which
    must be a compatible function. Here is an example of usage:
    
    >>> def f(*args, **kw):
    ...     return args[0]*2

    >>> def double(x):
    ...     pass

    >>> help(change_signature(f, double))
    Help on function double in module __main__:
    <BLANKLINE>
    double(x)
    <BLANKLINE>    
    """
    if model is None:
        return new.function(func.func_code, func.func_globals, func.func_name,
                            func.func_defaults, func.func_closure)
    else:
        return _decorate(func, lambda f, *a, **kw : f(*a,**kw),
                         getinfo(model))



from ez_setup import use_setuptools
use_setuptools()
from setuptools import setup, find_packages

version = "0.9.3"

setup(
    name="Pylons",
    version=version,
    description='Pylons Web Framework',
    long_description="""
Pylons
======

The Pylons web framework is aimed at making webapps and large programmatic website
development in Python easy. Several key points:

* A framework to make writing web applications in Python easy

* Inspired by Rails and TurboGears

* Utilizes a minimalist, component-based philosophy that makes it easy to expand on

* Built mainly as a customization of Myghty with Routes and Paste integration

* Harness existing knowledge about Python

Knowing Python makes Pylons easy
---------------------------------

Pylons makes it easy to expand on your knowledge of Python to master Pylons for web
development. Using a MVC style dispath, Python knowledge is used at various levels:

* The Controller is just a basic Python class, called for each request. Customizing the
  response is as easy as overriding __call__ to make your webapp work how you want.

* Myghty templating compiles directly to Python byte-code for speed and utilizes Python
  for template control rather than creating its own template syntax for "for, while, etc"

Current Status
---------------

Pylons %s described on this page is stable.

There is also an 
unstable `develoment version <http://pylonshq.com/svn/Pylons/trunk#egg=Pylons-dev>`_ 
of Pylons.

Download and Installation
-------------------------

Pylons can be installed with `Easy Install 
<http://peak.telecommunity.com/DevCenter/EasyInstall>`_ by typing::

    > easy_install Pylons 

Dependant packages are automatically installed from 
the `Pylons download page <http://pylonshq.com/download/>`_ .


"""%version,
    keywords='web wsgi framework sqlalchemy formencode myghty templates buffet',
    license='BSD',
    author='Ben Bangert, James Gardner',
    author_email='ben@groovie.org',
    url='http://www.pylonshq.com/',
    packages=find_packages(exclude=['ez_setup']),
    zip_safe=False,
    include_package_data=True,
    install_requires=[
        "Routes>=1.5.2", "WebHelpers>=0.2.2", "Beaker>=0.6.1",
        "Paste>=1.0", "PasteDeploy>=1.0", "PasteScript>=1.0",
        "Myghty>=1.1", "FormEncode>=0.6", "simplejson>=1.4", "nose>=0.9.0",
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    extras_require = {
        'pudge': ["docutils>=0.4", "elementtree>=1.2.6", "kid>=0.9", 
            "pudge==dev,>=0.1.1dev", "buildutils==dev,>=0.1.2dev",],
        'test': ["py>=0.8.0_alpha2"],
        'cheetah':["Cheetah>=1.0", "TurboCheetah>=0.9.5"],
        'kid':["kid>=0.9", "TurboKid==dev,>=0.9.1dev"],
        'full':[
            "docutils>=0.4", "elementtree>=1.2.6",
            "pudge==dev,>=0.1.1dev", "buildutils==dev,>=0.1.2dev",
            "py>=0.8.0_alpha2",
            "Cheetah>=1.0", "TurboCheetah>=0.9.5",
            "kid>=0.9", "TurboKid==dev,>=0.9.1dev",
        ],
    },
    entry_points="""
    [paste.paster_command]
    controller=pylons.commands:ControllerCommand
    shell=pylons.commands:ShellCommand

    [paste.paster_create_template]
    pylons=pylons.util:PylonsTemplate
    pylons_minimal=pylons.util:MinimalPylonsTemplate
    
    [distutils.commands]
    lang_extract = pylons.i18n:LangExtract
    lang_compile = pylons.i18n:LangCompile
    
    [python.templating.engines]
    pylonsmyghty = pylons.templating:MyghtyTemplatePlugin
    """,
)
